import { DateTime } from "luxon";
import { defaultAvailable, GanttViewType } from "./base";
import { CellDescriptor, DescriptorSet, IntervalDescriptor, TickDescriptor } from "./descriptors";


export class GanttViewWeek extends GanttViewType {

    constructor(date: DateTime) {
        super()

        this.start = date.startOf('week').startOf('day');
        this.viewDuration = this.start.endOf('week').endOf('day').diff(this.start).as('seconds');
        this.cellWidth = this.viewDuration / 7;
    }

    override setToday(): void {
        this.start = DateTime.now().startOf('week').startOf('day');
    }

    override calculateIntervalShift(action: "next" | "back"): void {
        if (action === 'next')
            this.start = this.start.plus({ days: 3.5 });
        else
            this.start = this.start.minus({ days: 3.5 });
    }

    override createDescriptorSet(): DescriptorSet { return { interval: this.createIntervals(), cell: this.createCells(), tick: this.createTicks() } };

    // get duration of a day in seconds
    dayDuration(day?: number): number {
        // check if first day is only a fraction of a day
        if (day === 0) {
            return Math.abs(this.start.diff(this.start.endOf('day')).as('seconds'));
        } else {
            return 24 * 3600;
        }
    };

    private createIntervals(): IntervalDescriptor[] {
        let weeks = [this.start.get('weekNumber')];
        let end = this.start.plus({ week: 1 }).minus({ seconds: 10 });
        if (!this.start.hasSame(end, 'week'))
            weeks.push(end.weekNumber)

        return weeks.map((item, index) => {
            return {
                title: `KW ${item}`,
                width: index === weeks.length - 1 ? '0px' : this.durationToWidth(this.start.endOf('week').diff(this.start).as('seconds'))
            }
        })
    }

    private createCells(): CellDescriptor[] {
        return Array.from({ length: 8 }, (_, day) => {
            let date = this.start.plus({ days: day });
            const duration = this.dayDuration(day);
            const sunOrHoliday = this.isDaySunOrHoliday(date);
            const weekend = this.isDayOnWeekend(date);

            return {
                title: date.toFormat('dd.MM'),
                subTitle: date.toFormat('EEE'),
                width: this.durationToWidth(duration),
                firstTickOffset: `${duration / this.cellWidth * 100 - 100}%`,
                sunOrHoliday,
                weekend,
                available: defaultAvailable.filter(item => ((!weekend && !sunOrHoliday) || (weekend && !sunOrHoliday) && item.onSaturday) || (sunOrHoliday && item.onSunOrHoliday)).map(item => {
                    return { offset: this.getOffset(date.startOf('day').plus({ seconds: item.start_day_sec })), duration: this.durationToWidth(item.duration_sec) }
                })
            }
        });
    }

    private createTicks(): TickDescriptor[] {
        // TODO make calculation more sane!!!
        let test = Math.floor(24 / Math.ceil((2 * 0.8 * this.globalBaseFontSize) / (this.dayDuration() * this.pixelPerSecond / 24)));

        return Array.from({ length: test }, (_, i) => i * 24 / test).map(item => {
            return <TickDescriptor>{
                title: item.toString(),
                width: this.durationToWidth(24 / test * 3600)
            }
        });
    }
}